package com.pcloud.common.utils.map;

/**
 * @描述：经纬度纠偏
 * @作者：lixing
 * @创建时间：2017年4月12日,下午17:30:19
 * @版本：1.0
 */
public class GpsCorrect {
    public static double pi = 3.14159265358979324;
    public static double a = 6378245.0;
    public static double ee = 0.00669342162296594323;

    public GpsCorrect ()
    {
        //
        // TODO: 在此处添加构造函数逻辑
        //
    }

    // 经纬度纠偏，适合谷歌、腾讯、高德地图系
    public static void transform(double wgLat, double wgLon, double[] latlng)
    {
        if (outOfChina(wgLat, wgLon))
        {
            latlng[0] = wgLat;
            latlng[1] = wgLon;
            return;
        }
        double dLat = transformLat(wgLon - 105.0, wgLat - 35.0);
        double dLon = transformLon(wgLon - 105.0, wgLat - 35.0);
        double radLat = wgLat / 180.0 * pi;
        double magic = Math.sin(radLat);
        magic = 1 - ee * magic * magic;
        double sqrtMagic = Math.sqrt(magic);
        dLat = (dLat * 180.0) / ((a * (1 - ee)) / (magic * sqrtMagic) * pi);
        dLon = (dLon * 180.0) / (a / sqrtMagic * Math.cos(radLat) * pi);
        latlng[0] = wgLat + dLat;
        latlng[1] = wgLon + dLon;
    }

    private static boolean outOfChina(double lat, double lon)
    {
        if (lon < 72.004 || lon > 137.8347) {
            return true;
        }
        if (lat < 0.8293 || lat > 55.8271) {
            return true;
        }
        return false;
    }

    private static double transformLat(double x, double y)
    {
        double ret = -100.0 + 2.0 * x + 3.0 * y + 0.2 * y * y + 0.1 * x * y + 0.2 * Math.sqrt(Math.abs(x));
        ret += (20.0 * Math.sin(6.0 * x * pi) + 20.0 * Math.sin(2.0 * x * pi)) * 2.0 / 3.0;
        ret += (20.0 * Math.sin(y * pi) + 40.0 * Math.sin(y / 3.0 * pi)) * 2.0 / 3.0;
        ret += (160.0 * Math.sin(y / 12.0 * pi) + 320 * Math.sin(y * pi / 30.0)) * 2.0 / 3.0;
        return ret;
    }

    private static double transformLon(double x, double y)
    {
        double ret = 300.0 + x + 2.0 * y + 0.1 * x * x + 0.1 * x * y + 0.1 * Math.sqrt(Math.abs(x));
        ret += (20.0 * Math.sin(6.0 * x * pi) + 20.0 * Math.sin(2.0 * x * pi)) * 2.0 / 3.0;
        ret += (20.0 * Math.sin(x * pi) + 40.0 * Math.sin(x / 3.0 * pi)) * 2.0 / 3.0;
        ret += (150.0 * Math.sin(x / 12.0 * pi) + 300.0 * Math.sin(x / 30.0 * pi)) * 2.0 / 3.0;
        return ret;
    }

    public static void main(String[] args){
        double wgLat=30.517384;
        double wgLon=114.337158;
        double[] latlng=new double[2];
        GpsCorrect.transform(wgLat,wgLon,latlng);
        System.out.println("转换前 GCJ-02 坐标："+wgLat+","+wgLon);
        System.out.println("转换后 GCJ-02 坐标："+latlng[0]+","+latlng[1]);

        double[] latlng2=new double[2];
        GpsConvert.bd_encrypt(wgLat, wgLon, latlng2);
        System.out.println("转换前 BD-09 坐标："+latlng2[0]+","+latlng2[1]);
        GpsConvert.bd_encrypt(latlng[0], latlng[1], latlng2);
        System.out.println("转换后 BD-09 坐标："+latlng2[0]+","+latlng2[1]);
    }
}
